#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>
#include <assert.h>
#include <memory.h>
#include <signal.h>
#include "PS2Defines.h"
#include "ps2maths.h"
#include "ps2matrix4x4.h"
#include "pad.h"
#include "sps2wrap.h"
#include "dma.h"
#include "texture.h"
#include "font.h"
#include "pipeline.h"
#include "vumanager.h"
#include "timer.h"
#include "ms3dmodel.h"
#include "terrain.h"
#include "hsfsocket.h"
#include "splines.h"


// These two external function pointers are exported by the VU code
// and we can now use them as pointers to the compiled VU data
extern "C" uint64 VU_vu1code_start;
extern "C" uint64 VU_vu1code_end;

bool g_bLoop = true;

void sig_handle(int sig)
{
	g_bLoop = false;
}

int main(void)
{	
	// Make sure these four lines are put first since some of the 
	// other classes need the managers to be initialised so they 
	// can pick up the correct data.	
	SPS2Manager.Initialise(4096);	// 4096 * 4K Pages = 16MB Total
	VIFStaticDMA.Initialise(3072);	// 1536 * 4K Pages = 6MB Static DMA
	VIFDynamicDMA.Initialise(256);	// 256 * 4K Pages * 2 Buffers =
									// 2MB Dynamic DMA
	Pipeline.Initialise();			// Initialise graphics pipline class
	
	
	// Initialise Lighting
	// Three lights and Ambient light
	//							Direction vector				  Colour	
	Pipeline.SetLight1(Vector4( 1.0f, 0.2f, 0.0f, 0.0f), Vector4(128.0f,128.0f,128.0f,0.0f));
	//Pipeline.SetLight2(Vector4(-1.0f,-0.2f, 0.0f, 0.0f), Vector4(0.0f,128.0f,0.0f,0.0f));
	//Pipeline.SetLight3(Vector4( 0.0f, 0.0f,-1.0f, 0.0f), Vector4(0.0f,0.0f,128.0f,0.0f));
	//                            Colour
	Pipeline.SetAmbient(Vector4(30.0f,30.0f,30.0f,0.0f));
	
	// Model to render
	CMs3dModel Model1, Model2, Road, Box16;
	CTexture ModelTex, Stars;
	
	// Spline Class
	CCRSpline CRSpline;
		
	// Load the models.
	if(!Model1.LoadModelData("gp2car.txt", false))
		g_bLoop = false;
	if(!Model2.LoadModelData("gp2car.txt", false))
		g_bLoop = false;
	if(!Road.LoadModelData("road6.txt", false))
		g_bLoop = false;
	if(!Box16.LoadModelData("box16.txt", false))
		g_bLoop = false;

	// Now load the model's texture.
	if(!ModelTex.LoadBitmap(Model1.GetTextureName()))
		g_bLoop = false;
		
	// Now load the Star texture.
	if(!Stars.LoadBitmap(Box16.GetTextureName()))
		g_bLoop = false;
	
	// Performance timer - call after SPS2Manager.Initialise()
	CTimer Timer;
	
	// Initialise control pad 0
	if(!pad_init(PAD_0, PAD_INIT_LOCK | PAD_INIT_ANALOGUE | PAD_INIT_PRESSURE))
	{
		printf("Failed to initialise control pad\n");
		pad_cleanup(PAD_0);
		exit(0);
	}	
	
	// Initialise the VU1 manager class
	CVU1MicroProgram VU1MicroCode(&VU_vu1code_start, &VU_vu1code_end);
	
	// Upload the microcode
	VU1MicroCode.Upload();
	
	// Register our signal function for every possible signal (i.e. ctrl + c)
	for(int sig=0; sig<128; sig++)
		signal(sig, sig_handle);
			
	// Set up the DMA packet to clear the screen. We want to clear to blue.
	SPS2Manager.InitScreenClear(0, 0x25, 0x50);
	
	
	// Load the font bitmap and data
	CFont Font;
	CTexture FontTex;
	if(!Font.Load("font.dat", true))
	{
		g_bLoop = false;
		printf("Can't load font.dat\n");
	}
	if(!FontTex.LoadBitmap("font.bmp", false, true))
	{
		printf("Can't load font.bmp\n");
		g_bLoop = false;
	}

	// Upload these once to VRAM since
	// they are not going to change
	FontTex.Upload(TEXBUF496);

	int Frame = 0;
	float fps = 0.0f;
	int Camera = 0;
	
	// Set default model positions
	Matrix4x4 matModel;
	matModel.Translation(15.0f,0,101.9);
	Model1.SetWorldMatrix(matModel);
	Model2.SetWorldMatrix(matModel);
	Road.SetWorldMatrix(Matrix4x4::IDENTITY);
			
	// The main Render loop
	while(g_bLoop)
	{
		//start the time
  		Timer.PrimeTimer();
  		
		VIFDynamicDMA.Fire();
		
		pad_update(PAD_0);
		
		// Check for exit condition
		if((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)) g_bLoop = false;
		
		// Process Human controlled car
		float Forward = 0.0f, Rotate = 0.0f;
		if(pad[0].buttons&PAD_CROSS)Forward = 0.5*pad[0].pressures[PAD_PCROSS];
		if(pad[0].buttons&PAD_RIGHT)Rotate = -0.1f*pad[0].pressures[PAD_PRIGHT];
		if(pad[0].buttons&PAD_LEFT)Rotate = 0.1f*pad[0].pressures[PAD_PLEFT];
		Model2.Move(Forward, Rotate);
		
		// Process whatever camera we are using
		if(pad[0].pressed & PAD_TRI)Camera++;	
		// Update the Camera and viewProjection matrices
		if(Camera%4 == 0)Pipeline.SetFollowCameraMatrix(Model1.GetWorld());
		if(Camera%4 == 1)Pipeline.SetFollowCameraMatrix(Model2.GetWorld());
		if(Camera%4 == 2)Pipeline.SetHeliCameraMatrix(Model1.GetWorld());
		if(Camera%4 == 3)
		{
			// Get any camera movements
			// Get any requested translations
			float Strafe =   pad[0].axes[PAD_AXIS_LX];
			float Advance =  pad[0].axes[PAD_AXIS_LY];
			float UpDown =   (pad[0].pressures[PAD_PL1] - pad[0].pressures[PAD_PL2]);
			
			// Get requested rotations
			float YRot = pad[0].axes[PAD_AXIS_RY];
			float XRot = pad[0].axes[PAD_AXIS_RX];
			
			// Update the Camera and viewProjection matrices
			Pipeline.Update(Strafe, Advance, UpDown, YRot, XRot);
		}
		

		// Advance the AI car along the spline
		CRSpline.AdvanceAlongSpline();
		
		// Get and set the new world matrix for the AI car
		Model1.SetWorldMatrix(CRSpline.GetMatrixAlongSpline());
		
		
		// DO all the rendering			
		SPS2Manager.BeginScene();
		
		
		// Select the model texture
		Stars.Upload(TEXBUF480);
		Stars.Select();		
		
		// Render skybox
		// Start the VIF direct mode.
		VIFDynamicDMA.StartDirect();
		VIFDynamicDMA.StartAD();
		
		VIFDynamicDMA.AddAD(TEST_SET(0, 0, 0, 0, 0, 0, 0, 1), TEST_1);		

		VIFDynamicDMA.EndAD();
		VIFDynamicDMA.EndDirect();	
		
		Matrix4x4 matSkyBox;
		matSkyBox = Matrix4x4::IDENTITY;
		Vector4 SkyBoxPosition = Pipeline.GetCameraPosition();
		matSkyBox(3,0) = SkyBoxPosition.x;
		matSkyBox(3,1) = SkyBoxPosition.y;
		matSkyBox(3,2) = SkyBoxPosition.z;
		Box16.SetWorldMatrix(matSkyBox);
		Box16.Render();
		
		VIFDynamicDMA.StartDirect();
		VIFDynamicDMA.StartAD();
		
		VIFDynamicDMA.AddAD(TEST_SET(0, 0, 0, 0, 0, 0, 1, 3), TEST_1);		

		VIFDynamicDMA.EndAD();
		VIFDynamicDMA.EndDirect();
		
		
		
		// Select the model texture
		ModelTex.Upload(TEXBUF480);
		ModelTex.Select();		
		
		// Render Model
		Model1.Render();
		Model2.Render();
		Road.Render();
			
		// Select the Font texture
		//FontTex.Upload(TEXBUF496);	
		FontTex.Select();
		
		// Render some text
		Font.printfC(  200, -240, 127, 127, 127, 127, "Frame: %d\nFPS: %.1f" , Frame++, fps);
		Font.printfL(  	-300, -240, 127, 127, 127, 127, 
						"Car Position (x, y, z) = (%3.1f, %3.1f, %3.1f)", 
						Model1.GetX(), Model1.GetY(), Model1.GetZ());
				
		//calculate the current frame rate in frames per second	
		fps = Timer.GetFPS();
		
		SPS2Manager.EndScene();	
		
		if(pad[0].pressed & PAD_R2)SPS2Manager.ScreenShot();		
	}

	pad_cleanup(PAD_0);
	
	return 0;
}

